'use strict';

let gulp   = require('gulp');
let phpcs  = require('gulp-phpcs');
let Elixir = require('laravel-elixir');

/*
 |----------------------------------------------------------------
 | PHP Code Sniffer
 |----------------------------------------------------------------
 |
 | This task triggers the entire PHP Code Sniffer
 | suite and shows notifications about the
 | success or failure of the gulp task.
 |
 */
Elixir.extend('phpcs', function initPHPCS (src, options) {
    // SETUP:       Notifications
    let notify = new Elixir.Notification();

    // SETUP:       Notification for errors
    let onError = function onError (err) {
        // REMOVE:      ANSI code from message
        err.message = err.message.replace(
            /[\u001b\u009b][[()#;?]*(?:[0-9]{1,4}(?:;[0-9]{0,4})*)?[0-9A-ORZcf-nqry=><]/g, '');

        notify.error(err, 'PHPCS Failed');
        this.emit('end');
    };


    // DEFAULT:       source
    let source = src || ['**/*.php', '!vendor/**/*', '!node_modules'];


    // ADD:         Task
    new Elixir.Task('phpcs', function taskPhpcs () {
        this.recordStep('Run PHP Code Sniffer over source files.');

        // DEFAULT:       settings
        let defaults = {
            bin:             './vendor/bin/phpcs',
            standard:        'PSR2',
            warningSeverity: 0,
            colors:          true,
        };
        let settings = Object.assign(defaults, options);


        // GULP:        Run it
        return gulp.src(source)
            // Validate files using PHP Code Sniffer
            .pipe(phpcs(settings))
            // Send to log reporter
            .pipe(phpcs.reporter('log'))
            // Send to errors to laravel-elixir
            .pipe(phpcs.reporter('fail'))
            // Send errors to elixir
            .on('error', onError);
    }).watch(source);
});
